@@ -62,15 +62,12 @@ void ClingoApp::initOptions(ProgramOptions::OptionContext& root) {
          "      plain : print rules as in plain output (prefix %%)\n"
          "      lparse: print rules as in lparse output (prefix %%%%)\n"
          "      all   : combines plain and lparse\n")
-        ("update-domains"           , flag(grOpts_.updateDomains = false), "Use top level assignment to cleanup domains")
         ("warn,W"                   , storeTo(grOpts_, parseWarning)->arg("<warn>")->composing(), "Enable/disable warnings:\n"
          "      [no-]atom-undefined:        a :- b.\n"
-         "      [no-]define-cyclic:         #const a=b. #const b=a.\n"
-         "      [no-]define-redefinition:   #const a=1. #const a=2.\n"
          "      [no-]file-included:         #include \"a.lp\". #include \"a.lp\".\n"
-         "      [no-]script-failure:        error in lua or python code\n"
          "      [no-]operation-undefined:   p(1/0).\n"
-         "      [no-]variable-unbounded:    $x > 10.\n")
+         "      [no-]variable-unbounded:    $x > 10.\n"
+         "      [no-]global-variable:       :- #count { X } = 1, X = 1.\n")
         ("rewrite-minimize"         , flag(grOpts_.rewriteMinimize = false), "Rewrite minimize constraints into rules")
         ("foobar,@4"                , storeTo(grOpts_.foobar, parseFoobar) , "Foobar")
         ;
