@@ -19,7 +19,17 @@
 //
 #include <clasp/solver.h>
 #include <clasp/clause.h>
-
+#if (defined(__cplusplus) && __cplusplus >= 201103L) || (defined(_MSC_VER) && _MSC_VER > 1500) || (defined(_LIBCPP_VERSION))
+#include <unordered_set>
+typedef std::unordered_set<Clasp::Constraint*> ConstraintSet;
+#else
+#if defined(_MSC_VER)
+#include <unordered_set>
+#else
+#include <tr1/unordered_set>
+#endif
+typedef std::tr1::unordered_set<Clasp::Constraint*> ConstraintSet;
+#endif
 #ifdef _MSC_VER
 #pragma warning (disable : 4996) // 'std::copy': Function call with parameters that may be unsafe
 #endif
@@ -157,7 +167,44 @@ void Solver::freeMem() {
 	ccMin_      = 0;
 	memUse_     = 0;
 }
-
+namespace {
+	struct InSet {
+		bool operator()(Constraint* c)        const { return set->find(c) != set->end(); }
+		bool operator()(const ClauseWatch& w) const { return (*this)(w.head);  }
+		bool operator()(const GenericWatch&w) const { return (*this)(w.con);  }
+		const ConstraintSet* set;
+	};
+}
+void Solver::destroyDB(ConstraintDB& db) {
+	bool lazy = db.size() > 100;
+	for (ConstraintDB::const_iterator it = db.begin(), end = db.end(); it != end; ++it) {
+		(*it)->destroy(this, !lazy);
+	}
+	if (lazy) {
+		ConstraintSet set(db.begin(), db.end());
+		InSet inSet = { &set };
+		for (Watches::iterator it = watches_.begin(), end = watches_.end(); it != end; ++it) {
+			if (it->left_size()) { it->shrink_left(std::remove_if(it->left_begin(), it->left_end(), inSet)); }
+			if (it->right_size()){ it->shrink_right(std::remove_if(it->right_begin(), it->right_end(), inSet)); }
+		}
+		for (uint32 i = 0, end = (uint32)levels_.size(); i != end; ++i) {
+			if (ConstraintDB* db = levels_[i].undo) {
+				db->erase(std::remove_if(db->begin(), db->end(), inSet), db->end());
+			}
+		}
+	}
+	db.clear();
+}
+void destroyDB(Solver::ConstraintDB& db, Solver* s, bool detach) {
+	if (s && detach) { 
+		s->destroyDB(db); 
+		return;
+	}
+	while (!db.empty()) {
+		db.back()->destroy(s, detach); 
+		db.pop_back();
+	}
+}
 SatPreprocessor*    Solver::satPrepro()     const { return shared_->satPrepro.get(); }
 const SolveParams&  Solver::searchConfig()  const { return shared_->configuration()->search(id()); }
 
@@ -282,6 +329,13 @@ bool Solver::endStep(uint32 top) {
 			else if (m)           { m->force(assign_.trail[tp++]); }
 			else                  { ++tp; }
 		}
+		if (x.var() && value(x.var()) != value_free) {
+			LitVec::iterator it = std::find(assign_.trail.begin(), assign_.trail.end(), trueLit(x.var()));
+			if (it != assign_.trail.end()) {
+				std::swap(*it, assign_.trail.back());
+				assign_.undoLast();
+			}
+		}
 		assign_.qReset();
 		assign_.setUnits(lastSimp_ = (uint32)assign_.trail.size()); 
 	}
@@ -453,8 +507,9 @@ bool Solver::clearAssumptions()  {
 
 void Solver::clearStopConflict() {
 	if (hasStopConflict()) {
-		levels_.root = conflict_[1].asUint();
-		assign_.front= conflict_[2].asUint();
+		levels_.root      = conflict_[1].asUint();
+		levels_.backtrack = conflict_[2].asUint();
+		assign_.front     = conflict_[3].asUint();
 		conflict_.clear();
 	}
 }
@@ -467,6 +522,8 @@ void Solver::setStopConflict() {
 		conflict_.push_back(negLit(0));
 		// remember the current root-level
 		conflict_.push_back(Literal::fromRep(rootLevel()));
+		// remember the current bt-level
+		conflict_.push_back(Literal::fromRep(backtrackLevel()));
 		// remember the current propagation queue
 		conflict_.push_back(Literal::fromRep(assign_.front));
 	}
@@ -1577,5 +1634,4 @@ ValueRep Solver::search(SearchLimits& limit, double rf) {
 	if (satPrepro()) { satPrepro()->extendModel(model, temp_); }
 	return value_true;
 }
-
 }
