@@ -1621,8 +1621,8 @@ DisjunctionRule::~DisjunctionRule() = default;
 
 void DisjunctionRule::report(Output::OutputBase &out) {
     bool undefined = false;
-    Value repr(complete.def.repr->eval(undefined));
-    auto &state = *complete.dom.domain.emplace(std::piecewise_construct, std::forward_as_tuple(repr), std::forward_as_tuple()).first;
+    Value repr(complete.repr->eval(undefined));
+    auto &state = *complete.dom.emplace(std::piecewise_construct, std::forward_as_tuple(repr), std::forward_as_tuple()).first;
     if (!state.second.defined() && !state.second.enqueued()) {
         state.second.enqueue();
         complete.todo.emplace_back(state.second);
@@ -1650,11 +1650,11 @@ DisjunctionAccumulateCond::~DisjunctionAccumulateCond() = default;
 
 void DisjunctionAccumulateCond::report(Output::OutputBase &) {
     bool undefined = false;
-    Value litRepr(complete.def.repr->eval(undefined));
+    Value litRepr(complete.repr->eval(undefined));
     Value condRepr(def.repr->eval(undefined));
     assert(!undefined);
 
-    auto &state = *complete.dom.domain.emplace(std::piecewise_construct, std::forward_as_tuple(litRepr), std::forward_as_tuple()).first;
+    auto &state = *complete.dom.emplace(std::piecewise_construct, std::forward_as_tuple(litRepr), std::forward_as_tuple()).first;
     Output::ULitVec cond;
     for (auto &x : lits) { 
         if(auto y = x->toOutput()) { 
@@ -1687,11 +1687,11 @@ DisjunctionAccumulateHead::~DisjunctionAccumulateHead() = default;
 
 void DisjunctionAccumulateHead::report(Output::OutputBase &) {
     bool undefined = false;
-    Value litRepr(complete.def.repr->eval(undefined));
+    Value litRepr(complete.repr->eval(undefined));
     assert(!undefined);
     Value headRepr(def.repr->eval(undefined));
     if (!undefined) {
-        auto &state = *complete.dom.domain.find(litRepr);
+        auto &state = *complete.dom.find(litRepr);
         auto &cond = *state.second.elems.find(headRepr.args()[2]);
         if (cond.heads.size() != 1 || !cond.heads.front().empty()) {
             Output::ULitVec head;
@@ -1723,25 +1723,25 @@ void DisjunctionAccumulateHead::report(Output::OutputBase &) {
 // {{{3 definition of DisjunctionComplete
 
 DisjunctionComplete::DisjunctionComplete(UTerm &&repr)
-: def(std::move(repr), &dom)
+: repr(std::move(repr))
 , inst(*this) { }
 
 UTerm DisjunctionComplete::emptyRepr() const {
     UTermVec args;
-    args.emplace_back(make_locatable<ValTerm>(def.repr->loc(), Value::createId("empty")));
-    args.emplace_back(get_clone(def.repr));
-    args.emplace_back(make_locatable<FunctionTerm>(def.repr->loc(), "", UTermVec()));
-    return make_locatable<FunctionTerm>(def.repr->loc(), "#accu", std::move(args));
+    args.emplace_back(make_locatable<ValTerm>(repr->loc(), Value::createId("empty")));
+    args.emplace_back(get_clone(repr));
+    args.emplace_back(make_locatable<FunctionTerm>(repr->loc(), "", UTermVec()));
+    return make_locatable<FunctionTerm>(repr->loc(), "#accu", std::move(args));
 }
 
 UTerm DisjunctionComplete::condRepr(unsigned elemIndex) const {
     UTermVec args;
-    args.emplace_back(make_locatable<ValTerm>(def.repr->loc(), Value::createId("cond")));
-    args.emplace_back(get_clone(def.repr));
+    args.emplace_back(make_locatable<ValTerm>(repr->loc(), Value::createId("cond")));
+    args.emplace_back(get_clone(repr));
     UTermVec condArgs = get_clone(locals[elemIndex]);
-    condArgs.emplace_back(make_locatable<ValTerm>(def.repr->loc(), Value::createNum(elemIndex)));
-    args.emplace_back(make_locatable<FunctionTerm>(def.repr->loc(), "", std::move(condArgs)));
-    return make_locatable<FunctionTerm>(def.repr->loc(), "#accu", std::move(args));
+    condArgs.emplace_back(make_locatable<ValTerm>(repr->loc(), Value::createNum(elemIndex)));
+    args.emplace_back(make_locatable<FunctionTerm>(repr->loc(), "", std::move(condArgs)));
+    return make_locatable<FunctionTerm>(repr->loc(), "#accu", std::move(args));
 }
 
 UTerm DisjunctionComplete::headRepr(unsigned elemIndex, int headIndex) const {
@@ -1749,22 +1749,22 @@ UTerm DisjunctionComplete::headRepr(unsigned elemIndex, int headIndex) const {
     if (headIndex >= 0) {
         UTermVec headArgs;
         headArgs.emplace_back(get_clone(heads[headIndex].repr));
-        args.emplace_back(make_locatable<FunctionTerm>(def.repr->loc(), "head", std::move(headArgs)));
+        args.emplace_back(make_locatable<FunctionTerm>(repr->loc(), "head", std::move(headArgs)));
     }
-    else { args.emplace_back(make_locatable<ValTerm>(def.repr->loc(), Value::createId("head"))); }
-    args.emplace_back(get_clone(def.repr));
+    else { args.emplace_back(make_locatable<ValTerm>(repr->loc(), Value::createId("head"))); }
+    args.emplace_back(get_clone(repr));
     UTermVec condArgs = get_clone(locals[elemIndex]);
-    condArgs.emplace_back(make_locatable<ValTerm>(def.repr->loc(), Value::createNum(elemIndex)));
-    args.emplace_back(make_locatable<FunctionTerm>(def.repr->loc(), "", std::move(condArgs)));
-    return make_locatable<FunctionTerm>(def.repr->loc(), "#accu", std::move(args));
+    condArgs.emplace_back(make_locatable<ValTerm>(repr->loc(), Value::createNum(elemIndex)));
+    args.emplace_back(make_locatable<FunctionTerm>(repr->loc(), "", std::move(condArgs)));
+    return make_locatable<FunctionTerm>(repr->loc(), "#accu", std::move(args));
 }
 
 UTerm DisjunctionComplete::accuRepr() const {
     UTermVec args;
-    args.emplace_back(make_locatable<VarTerm>(def.repr->loc(), "#Any1", std::make_shared<Value>(Value::createNum(0))));
-    args.emplace_back(get_clone(def.repr));
-    args.emplace_back(make_locatable<VarTerm>(def.repr->loc(), "#Any2", std::make_shared<Value>(Value::createNum(0))));
-    return make_locatable<FunctionTerm>(def.repr->loc(), "#accu", std::move(args));
+    args.emplace_back(make_locatable<VarTerm>(repr->loc(), "#Any1", std::make_shared<Value>(Value::createNum(0))));
+    args.emplace_back(get_clone(repr));
+    args.emplace_back(make_locatable<VarTerm>(repr->loc(), "#Any2", std::make_shared<Value>(Value::createNum(0))));
+    return make_locatable<FunctionTerm>(repr->loc(), "#accu", std::move(args));
 }
 
 bool DisjunctionComplete::isNormal() const {
@@ -1792,8 +1792,10 @@ void DisjunctionComplete::linearize(Scripts &, bool) {
     inst.finalize(Instantiator::DependVec{});
 } 
 
-void DisjunctionComplete::enqueue(Queue &q){
-    dom.init();
+void DisjunctionComplete::enqueue(Queue &q) {
+    for (HeadDefinition &x : heads) {
+        x.domain->init();
+    }
     q.enqueue(inst);
 }
 
