@@ -193,7 +193,7 @@ bool DisjointAggregate::operator==(BodyAggregate const &x) const {
 // {{{ definition of Aggregate::clone
 
 TupleBodyAggregate *TupleBodyAggregate::clone() const {
-    return make_locatable<TupleBodyAggregate>(loc(), naf, fun, get_clone(bounds), get_clone(elems)).release();
+    return make_locatable<TupleBodyAggregate>(loc(), naf, removedAssignment, translated, fun, get_clone(bounds), get_clone(elems)).release();
 }
 LitBodyAggregate *LitBodyAggregate::clone() const {
     return make_locatable<LitBodyAggregate>(loc(), naf, fun, get_clone(bounds), get_clone(elems)).release();
@@ -205,7 +205,7 @@ SimpleBodyLiteral *SimpleBodyLiteral::clone() const {
     return make_unique<SimpleBodyLiteral>(get_clone(lit)).release();
 }
 TupleHeadAggregate *TupleHeadAggregate::clone() const {
-    return make_locatable<TupleHeadAggregate>(loc(), fun, get_clone(bounds), get_clone(elems)).release();
+    return make_locatable<TupleHeadAggregate>(loc(), fun, translated, get_clone(bounds), get_clone(elems)).release();
 }
 LitHeadAggregate *LitHeadAggregate::clone() const {
     return make_locatable<LitHeadAggregate>(loc(), fun, get_clone(bounds), get_clone(elems)).release();
@@ -252,7 +252,7 @@ void TupleBodyAggregate::unpool(UBodyAggrVec &x, bool beforeRewrite) {
             e.emplace_back(std::move(elem));
         }
     }
-    auto f = [&](BoundVec &&y) { x.emplace_back(make_locatable<TupleBodyAggregate>(loc(), naf, fun, std::move(y), get_clone(e))); };
+    auto f = [&](BoundVec &&y) { x.emplace_back(make_locatable<TupleBodyAggregate>(loc(), naf, removedAssignment, translated, fun, std::move(y), get_clone(e))); };
     Term::unpool(bounds.begin(), bounds.end(), _unpool_bound, f);
 }
 
@@ -328,7 +328,7 @@ void TupleHeadAggregate::unpool(UHeadAggrVec &x, bool beforeRewrite) {
             e.emplace_back(std::move(elem));
         }
     }
-    auto f = [&](BoundVec &&y) { x.emplace_back(make_locatable<TupleHeadAggregate>(loc(), fun, std::move(y), get_clone(e))); };
+    auto f = [&](BoundVec &&y) { x.emplace_back(make_locatable<TupleHeadAggregate>(loc(), fun, translated, std::move(y), get_clone(e))); };
     Term::unpool(bounds.begin(), bounds.end(), _unpool_bound, f);
 }
 void LitHeadAggregate::unpool(UHeadAggrVec &x, bool beforeRewrite) {
@@ -491,7 +491,7 @@ bool TupleBodyAggregate::rewriteAggregates(UBodyAggrVec &aggr) {
     for (auto it = assign.begin(), ie = assign.end() - skip; it != ie; ++it) {
         BoundVec bound;
         bound.emplace_back(std::move(*it));
-        aggr.emplace_back(make_locatable<TupleBodyAggregate>(loc(), naf, fun, std::move(bound), get_clone(elems)));
+        aggr.emplace_back(make_locatable<TupleBodyAggregate>(loc(), naf, removedAssignment, translated, fun, std::move(bound), get_clone(elems)));
     }
     if (skip) { bounds.emplace_back(std::move(assign.back())); }
     return !bounds.empty();
@@ -507,7 +507,7 @@ bool LitBodyAggregate::rewriteAggregates(UBodyAggrVec &aggr) {
         lits.emplace_back(std::move(x.first));
         elems.emplace_back(std::move(tuple), std::move(lits));
     }
-    UBodyAggr x(make_locatable<TupleBodyAggregate>(loc(), naf, fun, std::move(bounds), std::move(elems)));
+    UBodyAggr x(make_locatable<TupleBodyAggregate>(loc(), naf, false, true, fun, std::move(bounds), std::move(elems)));
     if (x->rewriteAggregates(aggr)) { aggr.emplace_back(std::move(x)); }
     return false;
 }
@@ -529,7 +529,7 @@ bool DisjointAggregate::rewriteAggregates(UBodyAggrVec &) { return true; }
 UHeadAggr TupleHeadAggregate::rewriteAggregates(UBodyAggrVec &) { 
     for (auto &x : elems) {
         if (ULit shifted = std::get<1>(x)->shift(false)) {
-            // NOTE: FalseLiteral is a just bad name
+            // NOTE: FalseLiteral is a bad name
             std::get<1>(x) = make_locatable<FalseLiteral>(std::get<1>(x)->loc());
             std::get<2>(x).emplace_back(std::move(shifted));
         }
@@ -545,7 +545,7 @@ UHeadAggr LitHeadAggregate::rewriteAggregates(UBodyAggrVec &aggr) {
         x.first->toTuple(tuple, id);
         elems.emplace_back(std::move(tuple), std::move(x.first), std::move(x.second));
     }
-    UHeadAggr x(make_locatable<TupleHeadAggregate>(loc(), fun, std::move(bounds), std::move(elems)));
+    UHeadAggr x(make_locatable<TupleHeadAggregate>(loc(), fun, true, std::move(bounds), std::move(elems)));
     Term::replace(x, x->rewriteAggregates(aggr));
     return x;
 }
@@ -1003,18 +1003,36 @@ bool _aggr(ChkLvlVec &levels, BoundVec const &bounds, T const &f, bool bind) {
     return ret;
 }
 
+void warnGlobal(VarTermBoundVec &vars, bool warn) {
+    if (warn) {
+        auto ib = vars.begin(), ie = vars.end();
+        ie = std::remove_if(ib, ie, [](VarTermBoundVec::value_type const &a) { return a.first->level > 0; });
+        std::sort(ib, ie, [](VarTermBoundVec::value_type const &a, VarTermBoundVec::value_type const &b) { return a.first->name < b.first->name; });
+        ie = std::unique(ib, ie, [](VarTermBoundVec::value_type const &a, VarTermBoundVec::value_type const &b) { return a.first->name == b.first->name; });
+        for (auto it = ib; it != ie; ++it) {
+            GRINGO_REPORT(W_GLOBAL_VARIABLE)
+                << it->first->loc() << ": info: global variable in tuple of aggregate element:\n"
+                << "  " << it->first->name << "\n"
+                ;
+        }
+    }
+}
+
 } // namespace
 
 bool TupleBodyAggregate::check(ChkLvlVec &levels) const {
     auto f = [&]() -> bool {
         bool ret = true;
+        VarTermBoundVec vars;
         for (auto &y : elems) {
             levels.emplace_back(loc(), *this);
             _add(levels, y.first);
             _add(levels, y.second);
             ret = levels.back().check() && ret;
             levels.pop_back();
+            for (auto &term : y.first) { term->collect(vars, false); }
         }
+        warnGlobal(vars, !translated);
         return ret;
     };
     return _aggr(levels, bounds, f, naf == NAF::POS);
@@ -1064,6 +1082,7 @@ bool SimpleBodyLiteral::check(ChkLvlVec &levels) const {
 bool TupleHeadAggregate::check(ChkLvlVec &levels) const {
     auto f = [&]() -> bool {
         bool ret = true;
+        VarTermBoundVec vars;
         for (auto &y : elems) {
             levels.emplace_back(loc(), *this);
             _add(levels, std::get<0>(y));
@@ -1071,7 +1090,9 @@ bool TupleHeadAggregate::check(ChkLvlVec &levels) const {
             _add(levels, std::get<2>(y));
             ret = levels.back().check() && ret;
             levels.pop_back();
+            for (auto &term : std::get<0>(y)) { term->collect(vars, false); }
         }
+        warnGlobal(vars, !translated);
         return ret;
     };
     return _aggr(levels, bounds, f, false);
@@ -1520,7 +1541,7 @@ CreateHead SimpleHeadLiteral::toGround(ToGroundArg &x, Ground::UStmVec &, Ground
 
 // {{{ definition of *BodyAggregate::isAssignment
 
-bool TupleBodyAggregate::isAssignment() const { return !removedAssignment_ && bounds.size() == 1 && naf == NAF::POS && bounds.front().rel == Relation::EQ && bounds.front().bound->getInvertibility() == Term::INVERTIBLE; }
+bool TupleBodyAggregate::isAssignment() const { return !removedAssignment && bounds.size() == 1 && naf == NAF::POS && bounds.front().rel == Relation::EQ && bounds.front().bound->getInvertibility() == Term::INVERTIBLE; }
 bool LitBodyAggregate::isAssignment() const   { return false; }
 bool Conjunction::isAssignment() const        { return false; }
 bool SimpleBodyLiteral::isAssignment() const  { return false; }
@@ -1531,7 +1552,7 @@ bool DisjointAggregate::isAssignment() const  { return false; }
 
 void TupleBodyAggregate::removeAssignment() { 
     assert (isAssignment());
-    removedAssignment_ = true;
+    removedAssignment = true;
 }
 void LitBodyAggregate::removeAssignment()  { }
 void Conjunction::removeAssignment()       { }
@@ -1543,7 +1564,12 @@ void DisjointAggregate::removeAssignment() { }
 // {{{ definition of TupleBodyAggregate
 
 TupleBodyAggregate::TupleBodyAggregate(NAF naf, AggregateFunction fun, BoundVec &&bounds, BodyAggrElemVec &&elems)
+    : TupleBodyAggregate(naf, false, false, fun, std::move(bounds), std::move(elems)) { }
+
+TupleBodyAggregate::TupleBodyAggregate(NAF naf, bool removedAssignment, bool translated, AggregateFunction fun, BoundVec &&bounds, BodyAggrElemVec &&elems)
     : naf(naf)
+    , removedAssignment(removedAssignment)
+    , translated(translated)
     , fun(fun)
     , bounds(std::move(bounds))
     , elems(std::move(elems)) { }
@@ -1590,7 +1616,11 @@ SimpleBodyLiteral::~SimpleBodyLiteral() { }
 // {{{ definition of TupleHeadAggregate
 
 TupleHeadAggregate::TupleHeadAggregate(AggregateFunction fun, BoundVec &&bounds, HeadAggrElemVec &&elems)
+    : TupleHeadAggregate(fun, false, std::move(bounds), std::move(elems)) { }
+
+TupleHeadAggregate::TupleHeadAggregate(AggregateFunction fun, bool translated, BoundVec &&bounds, HeadAggrElemVec &&elems)
     : fun(fun)
+    , translated(translated)
     , bounds(std::move(bounds))
     , elems(std::move(elems)) { }
 
