@@ -194,7 +194,7 @@ bool handleError(lua_State *L, Location const &loc, int code, char const *desc,
                 << s << "\n"
                 ;
             if (!info) {
-                GRINGO_REPORT(ERROR) << msg.str();
+                GRINGO_REPORT(E_ERROR) << msg.str();
                 throw std::runtime_error("grounding stopped because of errors");
             }
             else {
@@ -203,7 +203,7 @@ bool handleError(lua_State *L, Location const &loc, int code, char const *desc,
             }
         }
         case LUA_ERRMEM: { 
-            GRINGO_REPORT(ERROR) << loc << ": error: lua interpreter ran out of memory" << "\n";
+            GRINGO_REPORT(E_ERROR) << loc << ": error: lua interpreter ran out of memory" << "\n";
             throw std::runtime_error("grounding stopped because of errors");
         }
     }
@@ -533,8 +533,7 @@ int newStatistics(lua_State *L, Statistics const *stats) {
             lua_newtable(L); // stack + 2
             for (char const *it = keys; *it; it+= strlen(it) + 1) {
                 if (strcmp(it, "__len") == 0) {
-                    char const *lenPrefix = lua_tostring(L, -1);
-                    int len = (int)protect<double>(L, [stats, lenPrefix]{ return stats->getStat(lenPrefix); });
+                    int len = (int)protect<double>(L, [stats, prefix]{ return stats->getStat((std::string(prefix) + "__len").c_str()); });
                     for (int i = 1; i <= len; ++i) {
                         lua_pushvalue(L, -2);
                         lua_pushinteger(L, i-1);
@@ -1062,6 +1061,12 @@ struct ControlWrap {
         }));
         return 1;
     }
+    static int cleanup_domains(lua_State *L) {
+        auto &ctl = get_self(L).ctl;
+        checkBlocked(L, ctl, "cleanup_domains");
+        protect<void>(L, [&ctl]() { ctl.cleanupDomains(); });
+        return 0;
+    }
     static int solve_async(lua_State *L) {
         auto &ctl = get_self(L).ctl;
         checkBlocked(L, ctl, "solve_async");
@@ -1255,6 +1260,7 @@ luaL_Reg ControlWrap::meta[] = {
     {"add", add},
     {"load", load},
     {"solve", solve},
+    {"cleanup_domains", cleanup_domains},
     {"solve_async", solve_async},
     {"solve_iter", solve_iter},
     {"get_const", get_const},
@@ -1278,7 +1284,8 @@ int cmpVal(lua_State *L) {
 int parseTerm(lua_State *L) {
     char const *str = luaL_checkstring(L, 1);
     Value val = protect<Value>(L, [str]() { return ControlWrap::module->parseValue(str); });
-    valToLua(L, val);
+    if (val.type() == Value::SPECIAL) { lua_pushnil(L); }
+    else { valToLua(L, val); }
     return 1;
 }
 
@@ -1573,7 +1580,7 @@ struct LuaImpl { };
 
 Lua::Lua(Gringo::GringoModule &) { }
 bool Lua::exec(Location const &loc, FWString) {
-    GRINGO_REPORT(ERROR)
+    GRINGO_REPORT(E_ERROR)
         << loc << ": error: gringo has been build without lua support\n"
         ;
     throw std::runtime_error("grounding stopped because of errors");
