@@ -144,6 +144,26 @@ bool PredicateLiteral::operator==(Literal const &x) const {
     PredicateLiteral const *t{dynamic_cast<PredicateLiteral const*>(&x)};
     return naf == t->naf && repr == t->repr;
 }
+std::pair<bool, TruthValue> PredicateLiteral::getTruth(AssignmentLookup assignment) {
+    if (repr->second.hasUid()) {
+        auto value = assignment(repr->second.uid());
+        if (naf == NAF::NOT) {
+            switch (value.second) {
+                case TruthValue::True: {
+                    value.second = TruthValue::False;
+                    break;
+                }
+                case TruthValue::False: {
+                    value.second = TruthValue::True;
+                    break;
+                }
+                default: { break; }
+            }
+        }
+        return value;
+    }
+    return {false, TruthValue::Open};
+}
 PredicateLiteral::~PredicateLiteral() { }
 
 // }}}
