@@ -413,7 +413,20 @@ struct DefaultLparseTranslator : LparseTranslator {
         }
         return get_clone(trueLit);
     }
-
+    virtual void simplify(AssignmentLookup assignment) {
+        minimize.erase(std::remove_if(minimize.begin(), minimize.end(), [&assignment](MinimizeList::value_type &elem) {
+            bool remove = false;
+            elem.second.erase(std::remove_if(elem.second.begin(), elem.second.end(), [&assignment,&remove](ULit &lit) {
+                auto value = lit->getTruth(assignment);
+                switch (value.second) {
+                    case TruthValue::True:  { return true; }
+                    case TruthValue::False: { remove = true; }
+                    default:                { return false; }
+                }
+            }), elem.second.end());
+            return remove;
+        }), minimize.end());
+    }
     virtual ~DefaultLparseTranslator() { }
 
     PredDomMap        &domains;
@@ -684,6 +697,9 @@ struct LparseHandler : StmHandler {
     virtual void atoms(int atomset, std::function<bool(unsigned)> const &isTrue, ValVec &atoms, OutputPredicates const &outPreds) {
         trans.atoms(atomset, isTrue, atoms, outPreds);
     }
+    virtual void simplify(AssignmentLookup assignment) {
+        trans.simplify(assignment);
+    }
     virtual ~LparseHandler() { }
 
     DefaultLparseTranslator trans;
@@ -709,6 +725,9 @@ struct LparsePlainHandler : StmHandler {
     }
     virtual void finish(OutputPredicates &) { trans.translate(); }
     virtual void atoms(int, std::function<bool(unsigned)> const &, ValVec &, OutputPredicates const &) { }
+    virtual void simplify(AssignmentLookup assignment) {
+        trans.simplify(assignment);
+    }
     virtual ~LparsePlainHandler() { }
 
     DefaultLparseTranslator trans;
@@ -737,6 +756,7 @@ struct PlainHandler : StmHandler {
         }
     }
     virtual void atoms(int, std::function<bool(unsigned)> const &, ValVec &, OutputPredicates const &) { }
+    virtual void simplify(AssignmentLookup) { }
     std::ostream &out;
 };
 
@@ -934,6 +954,50 @@ PredicateDomain::element_type *OutputBase::find2(Gringo::Value val) {
     }
     return nullptr;
 }
+std::pair<unsigned, unsigned> OutputBase::simplify(AssignmentLookup assignment) {
+    // TODO: would be nice to have this one in the statistics output
+    handler->simplify(assignment);
+    unsigned facts = 0;
+    unsigned deleted = 0;
+    for (auto &x : domains) {
+        unsigned offset = 0;
+        x.second.indices.clear();
+        x.second.fullIndices.clear();
+        x.second.exports.exports.erase(std::remove_if(x.second.exports.begin(), x.second.exports.end(), [&](Gringo::PredicateDomain::element_type &y) -> bool {
+            y.second.generation(offset);
+            if (y.second.hasUid()) {
+                auto value = assignment(y.second.uid());
+                if (!value.first) {
+                    switch (value.second) {
+                        case TruthValue::True: {
+                            // NOTE: externals cannot become facts here
+                            //       because they might get new definitions while grounding
+                            //       because there is no distinction between true and weak true
+                            //       these definitions might be skipped if a weak true external 
+                            //       is made a fact here
+                            if (!y.second.fact(false)) { ++facts; }
+                            y.second.setFact(true);
+                            break;
+                        }
+                        case TruthValue::False: {
+                            if (offset < x.second.exports.incOffset)  { --x.second.exports.incOffset; }
+                            if (offset < x.second.exports.showOffset) { --x.second.exports.showOffset; }
+                            x.second.domain.erase(y.first);
+                            ++deleted;
+                            return true;
+                        }
+                        default: { break; }
+                    }
+                }
+            }
+            ++offset;
+            return false;
+        }), x.second.exports.end());
+        x.second.exports.generation_     = 0;
+        x.second.exports.nextGeneration_ = x.second.exports.size();
+    }
+    return {facts, deleted};
+}
 
 // }}}
 
