@@ -39,7 +39,7 @@ void Defines::add(Location const &loc, FWString name, UTerm &&value, bool defaul
     if (it == defs_.end())                           { defs_.emplace(name, make_tuple(defaultDef, loc, std::move(value))); }
     else if (std::get<0>(it->second) && !defaultDef) { it->second = make_tuple(defaultDef, loc, std::move(value)); }
     else if (std::get<0>(it->second) || !defaultDef) {
-        GRINGO_REPORT(ERROR) 
+        GRINGO_REPORT(E_ERROR) 
             << loc << ": error: redefinition of constant:\n"
             << "  #const " << *name << "=" << *value << ".\n"
             << std::get<1>(it->second) << ": note: constant also defined here\n";
@@ -75,7 +75,7 @@ void Defines::init() {
                     << std::get<1>(x->data->second) << ": note: cycle involves definition:\n"
                     << "  #const " << *x->data->first << "=" << *std::get<2>(x->data->second) << ".\n";
             }
-            GRINGO_REPORT(ERROR) << msg.str();
+            GRINGO_REPORT(E_ERROR) << msg.str();
         }
         for (auto &x : scc) { Term::replace(std::get<2>(x->data->second), std::get<2>(x->data->second)->replace(*this, true)); }
     }
