@@ -22,6 +22,7 @@
 
 #include "tests/tests.hh"
 #include "gringo/lua.hh"
+#include "gringo_module.hh"
 
 namespace Gringo { namespace Test {
 
@@ -29,6 +30,7 @@ namespace Gringo { namespace Test {
 
 class TestLua : public CppUnit::TestFixture {
     CPPUNIT_TEST_SUITE(TestLua);
+        CPPUNIT_TEST(test_parse);
         CPPUNIT_TEST(test_callable);
         CPPUNIT_TEST(test_values);
         CPPUNIT_TEST(test_cmp);
@@ -39,6 +41,7 @@ public:
     virtual void tearDown();
 
     void test_values();
+    void test_parse();
     void test_cmp();
     void test_callable();
 
@@ -58,9 +61,23 @@ void TestLua::setUp() {
 void TestLua::tearDown() {
 }
 
+void TestLua::test_parse() {
+    Location loc("dummy", 1, 1, "dummy", 1, 1);
+    Lua lua(getTestModule());
+    lua.exec(loc,
+        "gringo = require(\"gringo\")\n"
+        "function get() return gringo.parse_term('1') end\n"
+        );
+    CPPUNIT_ASSERT_EQUAL(S("[1]"), to_string(lua.call(Any(), loc, "get", {})));
+    lua.exec(loc,
+        "gringo = require(\"gringo\")\n"
+        "function get() return gringo.parse_term('p(2+1)') end\n"
+        );
+    CPPUNIT_ASSERT_EQUAL(S("[p(3)]"), to_string(lua.call(Any(), loc, "get", {})));
+}
 void TestLua::test_values() {
     Location loc("dummy", 1, 1, "dummy", 1, 1);
-    Lua lua;
+    Lua lua(Gringo::Test::getTestModule());
     lua.exec(loc, 
         "gringo = require(\"gringo\")\n"
         "x = gringo.Fun(\"f\", {2, 3, 4})\n"
@@ -69,8 +86,8 @@ void TestLua::test_values() {
         "function none() return nil end\n"
         "values = {"
         "gringo.Fun(\"f\", {1, 2, 3}),"
-        "gringo.Sup(),"
-        "gringo.Inf(),"
+        "gringo.Sup,"
+        "gringo.Inf,"
         "gringo.Fun(\"id\"),"
         "gringo.Tuple({1, 2, 3}),"
         "123,"
@@ -84,10 +101,10 @@ void TestLua::test_values() {
     CPPUNIT_ASSERT_EQUAL(S("[f(1,2,3),#sup,#inf,id,(1,2,3),123,\"abc\",(2,3,4),\"f\"]"), to_string(lua.call(Any(), loc, "getValues", {})));
     {
         Gringo::Test::Messages msg;
-        CPPUNIT_ASSERT_EQUAL(S("[0]"), to_string(lua.call(Any(), loc, "none", {})));
+        CPPUNIT_ASSERT_EQUAL(S("[]"), to_string(lua.call(Any(), loc, "none", {})));
         CPPUNIT_ASSERT_EQUAL(S(
             "["
-            "dummy:1:1: warning: operation undefined, a zero is substituted:\n"
+            "dummy:1:1: info: operation undefined:\n"
             "  RuntimeError: cannot convert to value\n"
             "stack traceback:\n"
             "  [C]: in ?\n"
@@ -95,10 +112,10 @@ void TestLua::test_values() {
     }
     {
         Gringo::Test::Messages msg;
-        CPPUNIT_ASSERT_EQUAL(S("[0]"), to_string(lua.call(Any(), loc, "fail", {})));
+        CPPUNIT_ASSERT_EQUAL(S("[]"), to_string(lua.call(Any(), loc, "fail", {})));
         CPPUNIT_ASSERT_EQUAL(S(
             "["
-            "dummy:1:1: warning: operation undefined, a zero is substituted:\n"
+            "dummy:1:1: info: operation undefined:\n"
             "  RuntimeError: [string \"dummy:1:1\"]:4: cannot convert to value\n"
             "stack traceback:\n"
             "  [C]: in function 'Fun'\n"
@@ -108,10 +125,10 @@ void TestLua::test_values() {
     }
     {
         Gringo::Test::Messages msg;
-        lua.exec(loc, "(");
+        CPPUNIT_ASSERT_THROW(lua.exec(loc, "("), std::runtime_error);
         CPPUNIT_ASSERT_EQUAL(S(
             "["
-            "dummy:1:1: warning: parsing lua script failed:\n"
+            "dummy:1:1: error: parsing lua script failed:\n"
             "  SyntaxError: [string \"dummy:1:1\"]:1: unexpected symbol near <eof>\n"
             "]"), replace_all(IO::to_string(msg), "'<eof>'", "<eof>"));
     }
@@ -119,7 +136,7 @@ void TestLua::test_values() {
 
 void TestLua::test_cmp() {
     Location loc("dummy", 1, 1, "dummy", 1, 1);
-    Lua lua;
+    Lua lua(Gringo::Test::getTestModule());
     lua.exec(loc,
         "gringo = require(\"gringo\")\n"
         "function int(x) if x then return 1 else return 0 end end\n"
@@ -136,7 +153,7 @@ void TestLua::test_cmp() {
 
 void TestLua::test_callable() {
     Location loc("dummy", 1, 1, "dummy", 1, 1);
-    Lua lua;
+    Lua lua(Gringo::Test::getTestModule());
     lua.exec(loc, 
         "function a() end\n"
         "b = 1\n"
