@@ -39,6 +39,7 @@ class TestLparse : public CppUnit::TestFixture {
         CPPUNIT_TEST(test_headAggregateBug);
         CPPUNIT_TEST(test_headAggrPropagateBug);
         CPPUNIT_TEST(test_recHeadAggregateBug);
+        CPPUNIT_TEST(test_disjunctionBug);
         CPPUNIT_TEST(test_symTabBug);
         CPPUNIT_TEST(test_mutexBug);
         CPPUNIT_TEST(test_head);
@@ -83,6 +84,7 @@ public:
     void test_headAggregateBug();
     void test_headAggrPropagateBug();
     void test_recHeadAggregateBug();
+    void test_disjunctionBug();
     void test_symTabBug();
     void test_mutexBug();
     void test_head();
@@ -401,6 +403,17 @@ void TestLparse::test_recHeadAggregateBug() {
         , {"r("})));
 
 }
+void TestLparse::test_disjunctionBug() {
+    Gringo::Test::Messages msg;
+    CPPUNIT_ASSERT_EQUAL(
+        S("[]"),
+        IO::to_string(solve(
+            "b."
+            "c :- b."
+            "a :- c."
+            "c : d :- a."
+        )));
+}
 void TestLparse::test_aggregateNotNot() {
     CPPUNIT_ASSERT_EQUAL(
         S("[[p(2)],[p(2),p(3)],[p(2),p(3),p(4)],[p(2),p(3),p(4),p(5)],[p(2),p(3),p(5)],[p(2),p(4)],[p(2),p(4),p(5)],[p(2),p(5)],[p(4)],[p(4),p(5)]]"),
